/* This program creates a randomly generated map for VGA Planets 3.5.
   I have designed this program to be fully automated.  It loads itself,
   loads the xyplan.dat file, rondomizes the values, and returns a new file 
   It does not move the x value since Winplan likes that sum of all xs to equal
   99861.*/

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
++                                                                       ++
++ Version:		1.1                                                      ++
++                                                                       ++
++ Upadates:	June 9, 2000		Program now works with an array of   ++
++									planet objects.						 ++
++																		 ++
++                                  Added support for command-line       ++
++									arguments.							 ++
++                                                                       ++
++									Added support for other filenames    ++
++                                                                       ++
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <fstream.h>								// for the file input/output functions
#include <stdlib.h>
#include <time.h>

#define true 1
#define false 0

class TPlanet										// a class to hold the planet coordinates
{
public:
	short x_cor, y_cor, owner;						// the three features of each planet
	int FileRead(int pos)							// loads a single entry in the XYPLAN.DAT	
	{												// file into memory
		ifstream infile;							// create instance of ifstream class
		infile.open("XYPLAN.DAT", ios::binary);		// open XYPLAN.DAT
		infile.seekg( pos*sizeof(TPlanet) );		// point to the pos entry
		infile.read((char*)this, sizeof(*this) );	// read entry
		return 1;									// return true; it worked
	}
	int FileWrite(int index, char* filename)		// Saves the newly created file to
	{												// NewMap.dat
		ofstream outfile;							// create instance of ofstream class
		outfile.open(filename, ios::binary | ios::app);	// open file
		outfile.write( (char*)this, sizeof(*this) );	// write it
		return 1;									// it worked
	}
	int RandY(int n)				// Randomizes the value (y only: Winplan requires that all 
	{								// x values add up to a certain number.  To make sure that
									// they do, I don't change x at all.)
		srand(n * (unsigned)time(NULL));		// seed the generator
		y_cor = (rand() % 2000) + 1000;			// randomize y
		if(y_cor > 3000)			// if y is off the map, subtract 1000. This makes the lower
			y_cor =- 1000;			// end of the map a little more dense than the top. Oh well.
		return 1;									// it worked.
	}
	void clean()					// empties the file.  Just to make sure that there are no
	{								// trash entries
		y_cor = 0;					// y = 0
		owner = 0;					// there is no owner
	}
};	// end TPlanet class



//*********************************************************

/*	Structure of the arguments.  The First one is always the path to the exe.  The 
	Second is the name of the new map file.  The Third is the sub directory (i.e. VPWORKx)*/

void main(int argc, char* argv[])					// Enter main()
{
	TPlanet planet[500];		// an array of planets.
	
	//--------------------------------Set this block of code apart---------------------
	int fname_exists = 0;			// A bool value to tell whether there is a new filename
	if(argc >= 2)				// specified.  If there are 3 or more arguments...
		fname_exists = true;	// fname_exists is true; there is a filename
	else						// if there are less than three arguments...
		fname_exists = false;	// fname_exists is false; there isn't, use default
	//---------------------------------------------------------------------------------

	cout << "Welcome!  Let's Get Started!";
	for(int planet_num = 0; planet_num < 500; ++planet_num)	// a for loop to load the data
	{
		
		// load the next item (it starts at one and goes up)
		planet[planet_num].FileRead(planet_num);	
			
		// randomize the y coordinate
		planet[planet_num].RandY(planet_num);		
		
		// write the new coordinate to the file
		if(fname_exists)
			planet[planet_num].FileWrite(planet_num, argv[1]);
		else
			planet[planet_num].FileWrite(planet_num, "Newmap.dat");

		// verify
		cout << "Planet Number " << planet_num << " Edited" << endl;	

		// clear the object.  If you don't the x value will not move
		planet[planet_num].clean();	

	};	// end for loop
	return;
}	// end program
