//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include <stdlib.h>		// for atoi and itoa
#include "hullspecs.h"	// the header file that I created to access the HULLSPEC.DAT file
#include "truehull.h"   // the 'owners' section data
#include "About.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainSc *MainSc;
VHullSpec Hull;				// a VHullSpec object (see hullspecs.h>
VTrueHull Truehull;
int NodeIndex;				// and index to which item was selected from the tree view
//---------------------------------------------------------------------------
__fastcall TMainSc::TMainSc(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainSc::LoadTreeView(TObject *Sender)
{
    TreeList->Items->Clear();       // Clear all of the old data.  This keeps it
                                    // from loading the file twice.
    for (int i=0; i<105; i++)       // Start for loop which steps through each
    {                               // entry and adds it's name to the tree list
        Hull.readFile(i);           // Load an entry into the VHullSpec object
        AnsiString name = Hull.GetName();
        name.SetLength(30);
        TreeList->Items->Add(TreeList->Selected, name);   // Add a node
    }                               // to the tree list.
}

//---------------------------------------------------------------------------
void __fastcall TMainSc::TreeListChange(TObject *Sender, TTreeNode *Node)
{
   NodeIndex = Node->Index;            // Set the NodeIndex variable to the
   LoadHullData(Sender);               // Load the data from HULLSPEC.Dat
}
//---------------------------------------------------------------------------
void __fastcall TMainSc::LoadHullData(TObject *Sender)
{
    Hull.readFile(NodeIndex);           // open the first entry
    AnsiString name = Hull.GetName();
    name.SetLength(30);
    Name->Text = name;// set ship name
    char string[4];                     // string to hold temp values

/*The following lines cover the loading of the data which is in the VHullSpec object
into the program so that they will display in the boxes on the interface.  FuelTank is the name
of the edit box that holds the number for the size of the fuel tank.  Text is a member data item.
GetTankSize() is a function that I wrote which returns the size of the fuel tank for the selected
ship. */
    // Load the data.
    FuelBox->Text = itoa(Hull.GetTankSize(), string, 10);
    EngBox->Text = itoa(Hull.GetNumberOfEngines(), string, 10);
    BeamBox->Text = itoa(Hull.GetMaxBeams(), string, 10);
    TriBox->Text = itoa(Hull.GetTritanium(), string, 10);
    DurBox->Text = itoa(Hull.GetDuranium(), string, 10);
    MolyBox->Text = itoa(Hull.GetMolybdenum(), string, 10);
    CrewBox->Text = itoa(Hull.GetCrewSize(), string, 10);
    MassBox->Text = itoa(Hull.GetMass(), string, 10);
    TechBox->Text = itoa(Hull.GetTechLevel(), string, 10);
    CargoBox->Text = itoa(Hull.GetCargoSpace(), string, 10);
    BaysBox->Text = itoa(Hull.GetNumberOfFighterBays(), string, 10);
    TubeBox->Text = itoa(Hull.GetMaxTorpedoeL(), string, 10);
    CostBox->Text = itoa(Hull.GetCost(), string, 10);
    if(Hull.GetNumberOfFighterBays() > 0)
        AuxType->ItemIndex = 0;
    if(Hull.GetMaxTorpedoeL() > 0)
        AuxType->ItemIndex = 1;
    if(Hull.GetNumberOfFighterBays() == 0 && Hull.GetMaxTorpedoeL() == 0)
    {
        FightLabel->Enabled = false;
        TubeLabel->Enabled = false;
        BaysBox->Enabled = false;
        TubeBox->Enabled = false;
    }
    PictureBox->Text = itoa(Hull.GetPicture(), string, 10);
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++
    // Step through and check Truehull.dat for other owners
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++

    // First, clear old check data
    FedCheck->Checked = false;
    LizardCheck->Checked = false;
    BirdCheck->Checked = false;
    FascistCheck->Checked = false;
    PrivCheck->Checked = false;
    CyborgCheck->Checked = false;
    CrystalCheck->Checked = false;
    EmpireCheck->Checked = false;
    BotCheck->Checked = false;
    RebelCheck->Checked = false;
    ColoniesCheck->Checked = false;

    // Now check.
/*  in the following section, I copy the value in filenum, which is the index into the
HULLSPEC.DAT file, and add one to it to account for the 0 index.  Then I check the Truehull.dat
file for this hull model.  If it shows up under the right race, I set the check box to true
(see CheckTrueHullBox() below). If not, I leave it as false. */

int shipnum = NodeIndex + 1;
   for(int i=0; i<12; i++)
    {
        Truehull.readFile(i);
        for(int j=0; j<20; j++)
        {
            if(Truehull.getShipx(j) == shipnum)
                CheckTrueHullBox(Sender, i);
        } // end for
    } // end for
}  // end LoadDat

//---------------------------------------------------------------------------
void __fastcall TMainSc::CheckTrueHullBox(TObject* Sender, int raceNum)
{
    switch(raceNum)
    {
        case 0:
            FedCheck->Checked = true;
            break;
        case 1:
            LizardCheck->Checked = true;
            break;
        case 2:
            BirdCheck->Checked = true;
            break;
        case 3:
            FascistCheck->Checked = true;
            break;
        case 4:
            PrivCheck->Checked = true;
            break;
        case 5:
            CyborgCheck->Checked = true;
            break;
        case 6:
            CrystalCheck->Checked = true;
            break;
        case 7:
            EmpireCheck->Checked = true;
            break;
        case 8:
            BotCheck->Checked = true;
            break;
        case 9:
            RebelCheck->Checked = true;
            break;
        case 10:
            ColoniesCheck->Checked = true;
            break;
    };  // end switch
}

//----------------------------------------------------------------------------

void __fastcall TMainSc::AuxTypeClick(TObject *Sender)
{
    if(AuxType->ItemIndex)			// this function is supposed to check the
    {								// hull data and find out whether this ship has
        BaysBox->Enabled = false;	// fighter bays or torpedo tubes and deselect
        FightLabel->Enabled = false;// the false option.
        TubeBox->Enabled = true;
        TubeLabel->Enabled = true;
    }
    if(!AuxType->ItemIndex)
    {
        TubeBox->Enabled = false;
        TubeLabel->Enabled = false;
        BaysBox->Enabled = true;
        FightLabel->Enabled = true;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainSc::ButtonSaveClick(TObject *Sender)
{
    FSaveClick(MainSc);        // Save changes (speedbutton1 is the save button on the
									// toolbar
}
//---------------------------------------------------------------------------

void __fastcall TMainSc::FExitClick(TObject *Sender)
{
    Close();    
}
//---------------------------------------------------------------------------

void __fastcall TMainSc::FSaveClick(TObject *Sender)
{
    HullSave(Sender);  //Save the changes  (FSave is the save item on the main menu)
}
//---------------------------------------------------------------------------
void __fastcall TMainSc::HullSave(TObject* Sender)
{
    // Set the new name
    char* Buffer = new char[30];
    Name->GetTextBuf(Buffer, 30);
    Hull.SetName(Buffer);
    FuelBox->GetTextBuf(Buffer, 4);         // Get Fuel Tank Buffer
    Hull.SetTankSize(atoi(Buffer));         // Set New Size
    EngBox->GetTextBuf(Buffer, 4);          // Get Engine Number Buffer
    Hull.SetNumberOfEngines(atoi(Buffer));  // Set New Size
    BeamBox->GetTextBuf(Buffer, 4);
    Hull.SetMaxBeams(atoi(Buffer));
    TriBox->GetTextBuf(Buffer, 4);
    Hull.SetTritanium(atoi(Buffer));
    DurBox->GetTextBuf(Buffer, 4);
    Hull.SetDuranium(atoi(Buffer));
    MolyBox->GetTextBuf(Buffer, 4);
    Hull.SetMolybdenum(atoi(Buffer));
    CrewBox->GetTextBuf(Buffer, 4);
    Hull.SetCrewSize(atoi(Buffer));
    MassBox->GetTextBuf(Buffer, 4);
    Hull.SetMass(atoi(Buffer));
    TechBox->GetTextBuf(Buffer, 4);
    Hull.SetTechLevel(atoi(Buffer));
    CargoBox->GetTextBuf(Buffer, 4);
    Hull.SetCargo(atoi(Buffer));
    BaysBox->GetTextBuf(Buffer, 4);
    Hull.SetNumberOfBays(atoi(Buffer));
    TubeBox->GetTextBuf(Buffer, 4);
    Hull.SetMaxTorpedoeL(atoi(Buffer));
    CostBox->GetTextBuf(Buffer, 4);
    Hull.SetCost(atoi(Buffer));
    PictureBox->GetTextBuf(Buffer, 4);
    Hull.SetPicture(atoi(Buffer));

    Hull.writeFile(NodeIndex);      // Save Changes
    TreeList->Items->Item[NodeIndex]->Text = Hull.GetName();

    // save the Truehull section

    /*  this first section is supposed to create a list of which races have a particular
        ship.  The Truehull.dat file has the ships organized by race, this list is
        organized by ship and only has 11 entries */
    bool exist[11];
    int shipnum = NodeIndex + 1;
    for(int i=0; i<12; i++)
    {
        Truehull.readFile(i);
        for(int j=0; j<20; j++)
        {
            if(Truehull.getShipx(j) == shipnum)
                exist[i] = true;
        } // end for
        if (exist[i] != true)
            exist[i] = false;
    } // end for

    // if the box is checked and does not already contain the ship, save the change

    if (FedCheck->Checked == true && exist[0] == false) CheckSave(MainSc, 0);
    if (LizardCheck->Checked == true && exist[1] == false) CheckSave(MainSc, 1);
    if (BirdCheck->Checked == true && exist[2] == false) CheckSave(MainSc, 2);
    if (FascistCheck->Checked == true && exist[3] == false) CheckSave(MainSc, 3);
    if (PrivCheck->Checked == true && exist[4] == false) CheckSave(MainSc, 4);
    if (CyborgCheck->Checked == true && exist[5] == false) CheckSave(MainSc, 5);
    if (CrystalCheck->Checked == true && exist[6] == false) CheckSave(MainSc, 6);
    if (EmpireCheck->Checked == true && exist[7] == false) CheckSave(MainSc, 7);
    if (BotCheck->Checked == true && exist[8] == false) CheckSave(MainSc, 8);
    if (RebelCheck->Checked == true && exist[9] == false) CheckSave(MainSc, 9);
    if (ColoniesCheck->Checked == true && exist[10] == false) CheckSave(MainSc, 10);

    // if the box is not checked and the race does contain the ship, remove the entry
    if (FedCheck->Checked == false && exist[0] == true) RemoveCheck(MainSc, 0);
    if (LizardCheck->Checked == false && exist[1] == true) RemoveCheck(MainSc, 1);
    if (BirdCheck->Checked == false && exist[2] == true) RemoveCheck(MainSc, 2);
    if (FascistCheck->Checked == false && exist[3] == true) RemoveCheck(MainSc, 3);
    if (PrivCheck->Checked == false && exist[4] == true) RemoveCheck(MainSc, 4);
    if (CyborgCheck->Checked == false && exist[5] == true) RemoveCheck(MainSc, 5);
    if (CrystalCheck->Checked == false && exist[6] == true) RemoveCheck(MainSc, 6);
    if (EmpireCheck->Checked == false && exist[7] == true) RemoveCheck(MainSc, 7);
    if (BotCheck->Checked == false && exist[8] == true) RemoveCheck(MainSc, 8);
    if (RebelCheck->Checked == false && exist[9] == true) RemoveCheck(MainSc, 9);
    if (ColoniesCheck->Checked == false && exist[10] == true) RemoveCheck(MainSc, 10);
}
//---------------------------------------------------------------------------

void __fastcall TMainSc::HAboutClick(TObject *Sender)
{
    AboutSc->Show();
}
//---------------------------------------------------------------------------

void __fastcall TMainSc::FormCreate(TObject *Sender)
{
    LoadTreeView(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TMainSc::CheckSave(TObject* Sender, short racenum)
{
    char* string = new char;
    int shipnum = NodeIndex + 1;
    bool clear;
    int pos;

    /* This block creates the error message */
    /*--------------------------------------*/
    AnsiString num(racenum + 1);
    AnsiString mess("Error: Race ");
    AnsiString trailer(" has no empty spaces");
    mess = mess + num + trailer;
    /*--------------------------------------*/

    Truehull.readFile(racenum);
    for(int j=0; j<20; j++)
    {
        if(Truehull.getShipx(j) == 0)
        {
            clear = true;
            pos = j;
            j = 20;
        } // end if
    } // end for
    if (!clear)
    {
        Application->MessageBox (mess.c_str(), "Error saving file",
                                 MB_OKCANCEL + MB_DEFBUTTON1);
        return;
    } // end if
    if(clear)
        Truehull.setShipx(NodeIndex + 1, pos);
    Truehull.writeFile(racenum);
}

//-----------------------------------------------------------------------------

void __fastcall TMainSc::RemoveCheck(TObject* Sender, short racenum)
{
    Truehull.readFile(racenum);
    for(int j=0; j<20; j++)
    {
        if(Truehull.getShipx(j) == (NodeIndex + 1))
            Truehull.setShipx(0, j);
    }
    Truehull.writeFile(racenum);
}
