/*	This file contains the VBase class which controls access to the BDATAx.dat files in the
	VGAP directory.  It is structured like planets.h */


//	Only define fstream.h once.
#ifndef FSTREAM_H
#define FSTREAM_H

#include <fstream.h>
#endif

// Define Bases.h like this
#ifndef BASES_H
#define BASES_H

typedef short word;
typedef long dword;

class VBase
{
private:
	word base_id;
	word owner;
	word defence;
	word damage;
	word engine_tech;
	word hull_tech;
	word weapon_tech;
	word torpedo_tech;
	word engines_stored[9];
	word hulls_stored[20];
	word beams_stored[10];
	word torpedoeL_stored[10];
	word torpedoes_stored[10];
	word fighter_num;
	word recycle;				// the ID number of a ship to be recycled/repaired
	word operation;				// what to do with it (i.e. 0 = nothing, 1 = fix, 2 = rc
	word mission;				// base mission
	word build_queue;			// hull ID of ship to build
	word engine_count;
	word beam_type;
	word beam_count;
	word torpedoe_type;
	word torpedoe_count;
public:

	// The get list
	short GetBaseID() { return base_id; }
	short GetOwner() { return owner; }
	short GetDefence() { return defence; }
	short GetDamage() { return damage; }
	short GetEngineTech() { return engine_tech; }
	short GetHullTech() { return hull_tech; }
	short GetWeaponTech() { return weapon_tech; }
	short GetTorpedoeTech() { return torpedo_tech; }
	short GetEnginesStored(int index_num) { return engines_stored[index_num]; }
	short GetHullsStored(int index_num) { return hulls_stored[index_num]; }
	short GetBeamsStored(int index_num) { return beams_stored[index_num]; }
	short GetTorpedoeLStored(int index_num) {return torpedoeL_stored[index_num]; }
	short GetTorpedoesStored(int index_num) {return torpedoes_stored[index_num]; }
	short GetFighterNumber() {return fighter_num; }
	short GetRecycle() { return recycle; }
	short GetOperation() { return operation; }
	short GetMission() { return mission; }
	short GetBuildQueue() { return build_queue; }
	short GetEngineCount() { return engine_count; }
	short GetBeamType() { return beam_type; }
	short GetBeamCount() { return beam_count; }
	short GetTorpedoeType() { return torpedoe_type; }
	short GetTorpedoeCount() { return torpedoe_count; }

	// The set list
	void SetBaseID(short newBaseID) { base_id = newBaseID; }
	void SetOwner(short newOwner) { owner = newOwner; }
	void SetDefence(short newDefence) { defence = newDefence; }
	void SetDamage(short newDamage) { damage = newDamage; }
	void SetEngineTech( short newTech) { engine_tech = newTech; }
	void SetHullTech(short newTech) { hull_tech = newTech; }
	void SetWeaponTech(short newTech) { weapon_tech = newTech; }
	void SetTorpedoTech(short newTech) { torpedo_tech = newTech; }
	void SetEnginesStored(short in, short newCount) { engines_stored[in] = newCount; }
	void SetHullsStored(short in, short newCount) { hulls_stored[in] = newCount; }
	void SetBeamsStored(short in, short newCount) { beams_stored[in] = newCount; }
	void SetTorpedoeLStored(short in, short newCount) {torpedoeL_stored[in] = newCount; }
	void SetTorpedoesStored(short in, short newCount) {torpedoes_stored[in] = newCount; }
	void SetFighterNumber(short nc) { fighter_num = nc; }
	void SetRecycle(short nc) { recycle = nc; }
	void SetOperation(short nc) { operation = nc; }
	void SetMission(short nc) { mission = nc; }
	void SetBuildQueue(short nc) { build_queue = nc; }
	void SetEngineCount(short nc) { engine_count = nc; }
	void SetBeamType(short nc) { beam_type = nc; }
	void SetBeamCount(short nc) { beam_count = nc; }
	void SetTorpedoeType(short nc) { torpedoe_type = nc; }
	void SetTorpedoeCount(short nc) { torpedoe_count = nc; }

	// The following read/write the above data
	void readFile(int pos, char* file)
	{
		ifstream infile(file, ios::binary);
		infile.seekg((pos*sizeof(VBase))+4);
		infile.read((char*)this, sizeof(*this) );
	}

	void writeFile(int pos, char* file)
	{
		ofstream outfile(file, ios::binary);
		outfile.seekp((pos*sizeof(VBase))+4);
		outfile.write((char*)this, sizeof(*this));
	}
};

#endif

char* BDATA1 = "BDATA1.DAT";
char* BDATA2 = "BDATA2.DAT";
char* BDATA3 = "BDATA3.DAT";
char* BDATA4 = "BDATA4.DAT";
char* BDATA5 = "BDATA5.DAT";
char* BDATA6 = "BDATA6.DAT";
char* BDATA7 = "BDATA7.DAT";
char* BDATA8 = "BDATA8.DAT";
char* BDATA9 = "BDATA9.DAT";
char* BDATA10 = "BDATA10.DAT";
char* BDATA11 = "BDATA11.DAT";